# makepat.py

cnv = [0, 128, 192, 255]    # グレースケールの4階調（明度）
inputname = 'pattern.bmp'   # 入力ファイル名
outputname = 'pattern.h'    # 出力ファイル名

infile = open(inputname, 'rb')  # ファイルの読み込み
indata = infile.read()
infile.close()

header = int.from_bytes(indata[10:14], 'little') # ヘッダー長
width = int.from_bytes(indata[18:22], 'little')  # 幅
height = int.from_bytes(indata[22:26], 'little') # 高さ
depth = int.from_bytes(indata[28:30], 'little')  # カラービット

if depth != 4:
  print('ERROR:カラーが16色(4bit)ではありません')

pallete = []    # 16色カラーパレットを読み込み
for i in range(16):
  idx = 54 + (4 * i)
  color = int.from_bytes(indata[idx:idx+4], 'little')
  pallete.append(color)

comma = ' '
fbuf = '// '+ outputname +'\n'
fbuf += 'const unsigned char pattern[] = {\n'

for i in range(512):
  ofsx = (i & 15) * 8
  ofsy = int(i / 16) * 8
  for plane in range(2):
    for y  in range(8):
      tmp = 0
      for x  in range(8):
        ry = ((height - 1) - (y + ofsy))
        idx = int(((x + ofsx) + (ry * width)) / 2) + header
        if (x & 1)==0 :
          coloridx = (indata[idx] >> 4) & 0xf
        else:
          coloridx = indata[idx] & 0xf

        color = pallete[coloridx]   # 指定座標の色
        r = (color >> 16) & 0xff
        g = (color >> 8) & 0xff
        b = color & 0xff
        v = int((r + g + b) / 3)    # 256階調に変換
        for code in range(4):
          if v <= cnv[code]: break  # 4階調に変換

        if ((1 << plane) & code) != 0:
          tmp += (0x80 >> x)        # バイナリに変換

      fbuf += comma + '0x' + '{:02x}'.format(tmp)
      comma = ','

  fbuf += '\n'

fbuf += '};\n'
outfile = open(outputname, 'w')  # ファイルの出力
outfile.write(fbuf)
outfile.close()

